@@ -3,7 +3,7 @@ using System.Runtime.InteropServices;
 using Mono.Unix;
 
 namespace FSpot {
-	public class CDExport : GladeDialog {
+	public class CDExport : GladeDialog, FSpot.Extensions.IExporter {
 		IBrowsableCollection selection;
 
 		[Glade.Widget] Gtk.ScrolledWindow thumb_scrolledwindow;
@@ -20,7 +20,11 @@ namespace FSpot {
 		FSpot.ThreadProgressDialog progress_dialog;
 		System.Threading.Thread command_thread;
 
-		public CDExport (IBrowsableCollection selection) : base ("cd_export_dialog")
+		public CDExport () : base ("cd_export_dialog")
+		{
+		}
+
+		public void Run (IBrowsableCollection selection)
 		{
 			this.selection = selection;
 
@@ -80,11 +84,22 @@ namespace FSpot {
 	
 	                return dest;
 	        }
-	
+
+		void Clean ()
+		{
+			Gnome.Vfs.Uri target = dest.Clone ();
+			Gnome.Vfs.XferProgressCallback cb = new Gnome.Vfs.XferProgressCallback (Progress);
+			Gnome.Vfs.Xfer.XferDeleteList (new Gnome.Vfs.Uri [] {target}, Gnome.Vfs.XferErrorMode.Query, Gnome.Vfs.XferOptions.Recursive, cb);
+			
+		}
+
 		public void Transfer () {
 			try {
 				Gnome.Vfs.Result result = Gnome.Vfs.Result.Ok;
 
+				if (clean)
+					Clean ();
+
 				foreach (IBrowsableItem photo in selection.Items) {
 
  				//FIXME need to implement the uniquename as a filter	
@@ -120,10 +135,11 @@ namespace FSpot {
 					progress_dialog.Hide ();
 					system ("nautilus-cd-burner");
 				} else {
-					throw new System.Exception (System.String.Format ("{0}\n{1}\n{2}", 
+					throw new System.Exception (System.String.Format ("{0}{3}{1}{3}{2}", 
 											  progress_dialog.Message,
 											  Catalog.GetString ("Error While Transferring"), 
-											  result.ToString ()));
+											  result.ToString (),
+											  System.Environment.NewLine));
 				}
 
 			} catch (System.Exception e) {
