@@ -24,21 +24,29 @@ namespace FSpot {
 		const string ServicePath = "org.gnome.FSpot";
 		static ObjectPath CorePath = new ObjectPath ("/org/gnome/FSpot/Core");
 
-		public Core ()
+		public Core () : this (false)
+		{
+		}
+
+		public Core (bool nodb)
 		{
 			toplevels = new System.Collections.ArrayList ();
 			
-			// Load the database, upgrading/creating it as needed
-			string base_directory = FSpot.Global.BaseDirectory;
-			if (! File.Exists (base_directory))
-				Directory.CreateDirectory (base_directory);
-			
-			db = new Db ();
-			db.Init (Path.Combine (base_directory, "photos.db"), true);
 		}
 
 		public static Db Database {
-			get { return db; }
+			get { 
+				if (db == null) {
+					// Load the database, upgrading/creating it as needed
+					string base_directory = FSpot.Global.BaseDirectory;
+					if (! File.Exists (base_directory))
+						Directory.CreateDirectory (base_directory);
+					
+					db = new Db ();
+					db.Init (Path.Combine (base_directory, "photos.db"), true);
+				}
+				return db; 
+			}
 		}
 
 		public static ICore FindInstance ()
@@ -57,9 +65,9 @@ namespace FSpot {
 
 		public void RegisterServer ()
 		{
-			NameReply nameReply = Bus.Session.RequestName (ServicePath);
+			RequestNameReply nameReply = Bus.Session.RequestName (ServicePath, NameFlag.DoNotQueue);
 #if DEBUG_DBUS
-			Console.WriteLine("NameReply {0}", nameReply);
+			Console.WriteLine("RequestNameReply {0}", nameReply);
 #endif
 			Bus.Session.Register (ServicePath, CorePath, this);
 		}
@@ -95,7 +103,7 @@ namespace FSpot {
 		public MainWindow MainWindow {
 			get {
 				if (organizer == null) {
-					organizer = new MainWindow (db);
+					organizer = new MainWindow (Database);
 					Register (organizer.Window);
 				}
 				
@@ -137,15 +145,17 @@ namespace FSpot {
 				Tag tag;
 				
 				if (name != null)
-					tag = db.Tags.GetTagByName (name);
+					tag = Database.Tags.GetTagByName (name);
 				else {
 					int id = (int) Preferences.Get (Preferences.SCREENSAVER_TAG);
-					tag = db.Tags.GetTagById (id);
+					tag = Database.Tags.GetTagById (id);
 				}
 				
 				Photo [] photos;
 				if (tag != null)
-					photos = db.Photos.Query (new Tag [] { tag } );
+					photos = Database.Photos.Query (new Tag [] { tag } );
+ 				else if ((int) Preferences.Get (Preferences.SCREENSAVER_TAG) == 0)
+ 					photos = db.Photos.Query (new Tag [] {});
 				else
 					photos = new Photo [0];
 
@@ -163,13 +173,13 @@ namespace FSpot {
 					Gtk.HBox hbox = new Gtk.HBox ();
 					Gtk.VBox vbox = new Gtk.VBox ();
 
-					outer.PackStart (new Gtk.Label (""));
+					outer.PackStart (new Gtk.Label (String.Empty));
 					outer.PackStart (vbox, false, false, 0);
-					vbox.PackStart (new Gtk.Label (""));
+					vbox.PackStart (new Gtk.Label (String.Empty));
 					vbox.PackStart (hbox, false, false, 0);
 					hbox.PackStart (new Gtk.Image (Gtk.Stock.DialogWarning, Gtk.IconSize.Dialog),
 							false, false, 0);
-					outer.PackStart (new Gtk.Label (""));
+					outer.PackStart (new Gtk.Label (String.Empty));
 
 					string msg;
 					string long_msg;
@@ -181,8 +191,8 @@ namespace FSpot {
 											     "F-Spot preference dialog."), tag.Name);
 					} else {
 						msg = Catalog.GetString ("Search returned no results");
-						long_msg = Catalog.GetString ("The tag F-Spot is looking for does not exist. Try\n" +
-									      "selecting a different tag in the F-Spot preference\n" +
+						long_msg = Catalog.GetString ("The tag F-Spot is looking for does not exist. Try\n" + 
+									      "selecting a different tag in the F-Spot preference\n" + 
 									      "dialog.");
 					}
 
@@ -193,7 +203,7 @@ namespace FSpot {
 					long_label.Markup  = String.Format ("<small>{0}</small>", long_msg);
 
 					vbox.PackStart (long_label, false, false, 0);
-					vbox.PackStart (new Gtk.Label (""));
+					vbox.PackStart (new Gtk.Label (String.Empty));
 
 					window.Add (outer);
 					SetStyle (label);
@@ -227,7 +237,11 @@ namespace FSpot {
 
 		public void Shutdown ()
 		{
-			System.Environment.Exit (0);
+			try {
+				MainWindow.Toplevel.Close ();
+			} catch {
+				System.Environment.Exit (0);
+			}
 		}
 
 		public void Register (Gtk.Window window)
