@@ -26,13 +26,13 @@ public class ImportCommand : FSpot.GladeDialog {
 		public BrowseSource ()
 		{
 			this.Name = Catalog.GetString ("Select Folder");
-			this.Icon = PixbufUtils.LoadThemeIcon ("stock_folder", 32);
+			this.Icon = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, "stock_folder", 32, (Gtk.IconLookupFlags)0);
 		}
 
 		public BrowseSource (string name, string icon)
 		{
 			this.Name = name;
-			this.Icon = PixbufUtils.LoadThemeIcon (icon, 32);
+			this.Icon = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, icon, 32, (Gtk.IconLookupFlags)0);
 		}
 	}
 
@@ -43,12 +43,12 @@ public class ImportCommand : FSpot.GladeDialog {
 		{ 
 			string [] components = uri.Split (new char [] { '/' });
 			this.Name = components [components.Length - 1];
-			if (this.Name == "")
+			if (this.Name == String.Empty)
 				this.Name = components [components.Length - 2];
 
 			this.uri = uri;
 			
-			this.Icon = PixbufUtils.LoadThemeIcon ("stock_folder", 32);
+			this.Icon = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, "stock_folder", 32, (Gtk.IconLookupFlags)0);
 		}
 
 		public virtual bool Contains (string path)
@@ -77,13 +77,13 @@ public class ImportCommand : FSpot.GladeDialog {
 			uri = mount_point;
 			
                         if (this.Icon == null)
-				this.Icon = PixbufUtils.LoadThemeIcon (vol.Icon, 32);
+				this.Icon = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, vol.Icon, 32, (Gtk.IconLookupFlags)0);
 			
 			if (this.IsIPodPhoto)
-				this.Icon = PixbufUtils.LoadThemeIcon ("gnome-dev-ipod", 32);
+				this.Icon = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, "gnome-dev-ipod", 32, (Gtk.IconLookupFlags)0);
 
 			if (this.Icon == null && this.IsCamera)
-				this.Icon = PixbufUtils.LoadThemeIcon ("gnome-dev-media-cf", 32);
+				this.Icon = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, "gnome-dev-media-cf", 32, (Gtk.IconLookupFlags)0);
 
 			try {
 				if (this.Icon == null)
@@ -128,10 +128,10 @@ public class ImportCommand : FSpot.GladeDialog {
 			this.Drive = drive;
 
 			if (drive.IsMounted) {
-				this.Icon = PixbufUtils.LoadThemeIcon (drive.MountedVolume.Icon, 32);
+				this.Icon = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, drive.MountedVolume.Icon, 32, (Gtk.IconLookupFlags)0);
 				//this.Sensitive = drive.MountedVolume.IsMounted;
 			} else {
-				this.Icon = PixbufUtils.LoadThemeIcon (drive.Icon, 32);
+				this.Icon = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, drive.Icon, 32, (Gtk.IconLookupFlags)0);
 			}
 		}
 	}
@@ -150,9 +150,9 @@ public class ImportCommand : FSpot.GladeDialog {
 #else
 			this.Name = String.Format ("{0}", cam.CameraList.GetName (index));
 #endif
-			this.Icon = PixbufUtils.LoadThemeIcon ("gnome-dev-camera", 32);
+			this.Icon = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, "gnome-dev-camera", 32, (Gtk.IconLookupFlags)0);
 			if (this.Icon == null)
-				this.Icon = PixbufUtils.LoadThemeIcon ("gnome-dev-media-cf", 32);
+				this.Icon = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, "gnome-dev-media-cf", 32, (Gtk.IconLookupFlags)0);
 		}
 
 		public string Port {
@@ -291,11 +291,10 @@ public class ImportCommand : FSpot.GladeDialog {
 		}
 	}		
 	
-	[Glade.Widget] Gtk.OptionMenu tag_option_menu;
+	[Glade.Widget] Gtk.HBox tagentry_box;
 	[Glade.Widget] Gtk.OptionMenu source_option_menu;
 	[Glade.Widget] Gtk.ScrolledWindow icon_scrolled;
 	[Glade.Widget] Gtk.ScrolledWindow photo_scrolled;
-	[Glade.Widget] Gtk.CheckButton attach_check;
 	[Glade.Widget] Gtk.CheckButton recurse_check;
 	[Glade.Widget] Gtk.CheckButton copy_check;
 	[Glade.Widget] Gtk.Button ok_button;
@@ -303,8 +302,11 @@ public class ImportCommand : FSpot.GladeDialog {
 	[Glade.Widget] Gtk.Label tag_label;
 	[Glade.Widget] Gtk.EventBox frame_eventbox;
 	[Glade.Widget] ProgressBar progress_bar;
+	[Glade.Widget] Gtk.HPaned import_hpaned;
 	
-	Tag tag_selected;
+	ArrayList tags_selected;
+
+	FSpot.Widgets.TagEntry tag_entry;
 
 	Gtk.Window main_window;
 	FSpot.PhotoList collection;
@@ -499,11 +501,6 @@ public class ImportCommand : FSpot.GladeDialog {
 		importer = null;
 	}
 	
-	public void HandleTagToggled (object o, EventArgs args) 
-	{
-		tag_option_menu.Sensitive = attach_check.Active;
-	}
-
 	public void HandleImportBrowse (object o, EventArgs args) 
 	{
 		string path = ChoosePath ();
@@ -542,12 +539,12 @@ public class ImportCommand : FSpot.GladeDialog {
 		import_path = path;
 	}
 
-	private void HandleTagMenuSelected (Tag t) 
-	{
-		tag_selected = t;
-		//tag_image.Pixbuf = t.Icon;
-		//tag_label.Text = t.Name;
-	}
+//	private void HandleTagMenuSelected (Tag t) 
+//	{
+//		tag_selected = t;
+//		//tag_image.Pixbuf = t.Icon;
+//		//tag_label.Text = t.Name;
+//	}
 
 	private void HandleRecurseToggled (object sender, System.EventArgs args)
 	{
@@ -566,8 +563,12 @@ public class ImportCommand : FSpot.GladeDialog {
 		this.Dialog.WindowPosition = Gtk.WindowPosition.CenterOnParent;
 		this.Dialog.Response += HandleDialogResponse;
 
-		CreateTagMenu ();
-		
+ 		if ((int) FSpot.Preferences.Get (FSpot.Preferences.IMPORT_WINDOW_WIDTH) > 0)
+ 			this.Dialog.Resize ((int) FSpot.Preferences.Get (FSpot.Preferences.IMPORT_WINDOW_WIDTH), (int) FSpot.Preferences.Get(FSpot.Preferences.IMPORT_WINDOW_HEIGHT));
+
+ 		if ((int) FSpot.Preferences.Get (FSpot.Preferences.IMPORT_WINDOW_PANE_POSITION) > 0)
+			import_hpaned.Position = (int) FSpot.Preferences.Get (FSpot.Preferences.IMPORT_WINDOW_PANE_POSITION);
+
 	        AllowFinish = false;
 		
 		this.Dialog.DefaultResponse = ResponseType.Ok;
@@ -582,7 +583,7 @@ public class ImportCommand : FSpot.GladeDialog {
 		collection = new FSpot.PhotoList (new Photo [0]);
 		tray = new FSpot.ScalingIconView (collection);
 		tray.Selection.Changed += HandleTraySelectionChanged;
-		icon_scrolled.SetSizeRequest (200, 200);
+		icon_scrolled.SetSizeRequest (400, 200);
 		icon_scrolled.Add (tray);
 		//icon_scrolled.Visible = false;
 		tray.DisplayTags = false;
@@ -597,16 +598,17 @@ public class ImportCommand : FSpot.GladeDialog {
 		FSpot.Global.ModifyColors (photo_scrolled);
 		FSpot.Global.ModifyColors (photo_view);
 
-		photo_view.Pixbuf = PixbufUtils.LoadFromAssembly ("f-spot-48.png");
+		photo_view.Pixbuf = FSpot.Global.IconTheme.LoadIcon ("f-spot", 48, (Gtk.IconLookupFlags)0);
 		photo_view.Fit = true;
 			
-		tag_selected = null;
-		if (attach_check != null) {
-			attach_check.Toggled += HandleTagToggled;
-			HandleTagToggled (null, null);
-		}				
+		tag_entry = new FSpot.Widgets.TagEntry (MainWindow.Toplevel.Database.Tags, false);
+		tag_entry.UpdateFromTagNames (new string []{});
+		tagentry_box.Add (tag_entry);
+
+		tag_entry.Show ();
 
 		this.Dialog.Show ();
+
 		//source_option_menu.Changed += HandleSourceChanged;
 		if (path != null) {
 			SetImportPath (path);
@@ -650,20 +652,28 @@ public class ImportCommand : FSpot.GladeDialog {
 		}
 
 		if (response == ResponseType.Ok) {
+			this.UpdateTagStore (tag_entry.GetTypedTagNames ());
 			this.Finish ();
 
-			if (attach_check.Active && tag_selected != null) {
+			if (tags_selected != null && tags_selected.Count > 0) {
 				for (int i = 0; i < collection.Count; i++) {
 					Photo p = collection [i] as Photo;
 					
 					if (p == null)
 						continue;
 					
-					p.AddTag (tag_selected);
+					p.AddTag ((Tag [])tags_selected.ToArray(typeof(Tag)));
 					store.Commit (p);
 				}
 			}
 
+			int width, height;
+			this.Dialog.GetSize (out width, out height);
+
+			FSpot.Preferences.Set (FSpot.Preferences.IMPORT_WINDOW_WIDTH, width);
+			FSpot.Preferences.Set (FSpot.Preferences.IMPORT_WINDOW_HEIGHT, height);
+			FSpot.Preferences.Set (FSpot.Preferences.IMPORT_WINDOW_PANE_POSITION, import_hpaned.Position);
+
 			this.Dialog.Destroy ();
 			return collection.Count;
 		} else {
@@ -673,30 +683,34 @@ public class ImportCommand : FSpot.GladeDialog {
 		}
 	}
 
-	private void CreateTagMenu ()
+	private void UpdateTagStore (string [] new_tags)
 	{
-		TagMenu tagmenu = new TagMenu (null, MainWindow.Toplevel.Database.Tags);
-		tagmenu.NewTagHandler = HandleNewTagSelected;
+		if (new_tags == null || new_tags.Length == 0)
+			return;
 
-		tagmenu.Append (new MenuItem (Catalog.GetString ("Select Tag")));
-		
-		tagmenu.Populate (true);
-		
-		tagmenu.TagSelected += HandleTagMenuSelected;
-		
-		tagmenu.ShowAll ();
-		tag_option_menu.Menu = tagmenu;
-	}
+		tags_selected = new ArrayList ();
+		Db db = MainWindow.Toplevel.Database;	
+		db.BeginTransaction ();
+		foreach (string tagname in new_tags) {
+			Tag t = db.Tags.GetTagByName (tagname);
+			if (t == null) {
+				Category default_category = db.Tags.GetTagByName (Catalog.GetString ("Import Tags")) as Category;
+				if (default_category == null) {
+					default_category = db.Tags.CreateCategory (null, Catalog.GetString ("Import Tags"));
+					default_category.ThemeIconName = "f-spot-imported-xmp-tags.png"; 
+				}
+				t = db.Tags.CreateCategory (default_category, tagname) as Tag;
+				db.Tags.Commit (t);
+			}
 
-	private void HandleNewTagSelected (object sender, EventArgs args)
-	{
-		Tag new_tag = MainWindow.Toplevel.CreateTag (Dialog, null);
-
-		if (new_tag != null) {
-			CreateTagMenu ();
-			tag_option_menu.SetHistory ((uint) (tag_option_menu.Menu as TagMenu).GetPosition (new_tag));
-			tag_selected = new_tag;
+			tags_selected.Add (t);
 		}
+		db.CommitTransaction ();
+		
+		ArrayList tagnames = new ArrayList ();
+		foreach (Tag t in tags_selected)
+			tagnames.Add (t.Name);
+		tag_entry.UpdateFromTagNames ((string [])tagnames.ToArray(typeof(string)));
 	}
 
 	public void Cancel ()
