@@ -13,6 +13,8 @@ using System.Text.RegularExpressions;
 using System.Web;
 using System.Web.Mail;
 
+using Banshee.Kernel;
+
 using FSpot;
 using FSpot.Widgets;
 using LibGPhoto2;
@@ -46,6 +48,7 @@ public class MainWindow {
 	[Glade.Widget] MenuItem delete_version_menu_item;
 	[Glade.Widget] MenuItem rename_version_menu_item;
 	
+	[Glade.Widget] MenuItem tools;
 	[Glade.Widget] MenuItem export;
 	[Glade.Widget] MenuItem print;
 	[Glade.Widget] MenuItem send_mail;
@@ -80,6 +83,11 @@ public class MainWindow {
 	[Glade.Widget] MenuItem zoom_in;
 	[Glade.Widget] MenuItem zoom_out;
 
+	[Glade.Widget] RadioMenuItem tag_icon_hidden;
+	[Glade.Widget] RadioMenuItem tag_icon_small;
+	[Glade.Widget] RadioMenuItem tag_icon_medium;
+	[Glade.Widget] RadioMenuItem tag_icon_large;
+
 	[Glade.Widget] RadioMenuItem month;
 	[Glade.Widget] RadioMenuItem directory;
 	[Glade.Widget] CheckMenuItem reverse_order;
@@ -93,6 +101,10 @@ public class MainWindow {
 
 	[Glade.Widget] CheckMenuItem find_untagged;
 	
+	[Glade.Widget] MenuItem last_roll;
+	[Glade.Widget] MenuItem select_rolls;
+	[Glade.Widget] MenuItem clear_roll_filter;	
+	
 	// Tags
 	[Glade.Widget] MenuItem edit_selected_tag;
 	[Glade.Widget] MenuItem delete_selected_tag;
@@ -105,9 +117,6 @@ public class MainWindow {
 
 	[Glade.Widget] VBox info_vbox;
 
-	[Glade.Widget] Gtk.Image near_image;
-	[Glade.Widget] Gtk.Image far_image;
-
 	[Glade.Widget] Gtk.HBox tagbar;
 	[Glade.Widget] Gtk.VBox tag_entry_container;
 	TagEntry tag_entry;
@@ -118,8 +127,8 @@ public class MainWindow {
 
 	PhotoVersionMenu versions_submenu;
 
-	Gtk.ToggleButton browse_button;
-	Gtk.ToggleButton edit_button;
+	Gtk.ToggleToolButton browse_button;
+	Gtk.ToggleToolButton edit_button;
 
 	InfoBox info_box;
 	FSpot.InfoDisplay info_display;
@@ -131,14 +140,18 @@ public class MainWindow {
 	FSpot.QueryWidget query_widget;
 	MainSelection selection;
 	
-	FSpot.Delay slide_delay;
-
-	Widget rl_button;
-	Widget rr_button;
+	ToolButton rl_button;
+	ToolButton rr_button;
 	
 	ModeType view_mode;
 	bool write_metadata = false;
 
+	// Tag Icon Sizes
+	public int TagsIconSize {
+		get { return (int) Tag.TagIconSize; }
+		set { Tag.TagIconSize = (Tag.IconSize) value; }
+	}
+
 	// Drag and Drop
 	public enum TargetType {
 		UriList,
@@ -172,7 +185,14 @@ public class MainWindow {
 
 	const int PHOTO_IDX_NONE = -1;
 
-	private static Gtk.Tooltips toolTips = new Gtk.Tooltips ();
+	private static Gtk.Tooltips toolTips;
+	public static Gtk.Tooltips ToolTips {
+		get {
+			if (toolTips == null)
+				toolTips = new Gtk.Tooltips ();
+			return toolTips;
+		}
+	}
 
 	//
 	// Public Properties
@@ -219,32 +239,54 @@ public class MainWindow {
 		LoadPreference (Preferences.SIDEBAR_POSITION);
 		LoadPreference (Preferences.METADATA_EMBED_IN_IMAGE);
 		
-		slide_delay = new FSpot.Delay (new GLib.IdleHandler (SlideShow));
-		
 		toolbar = new Gtk.Toolbar ();
 		toolbar_vbox.PackStart (toolbar);
-		
-		rl_button = GtkUtil.MakeToolbarButton (toolbar, "f-spot-rotate-270", new System.EventHandler (HandleRotate270Command));
-		rr_button = GtkUtil.MakeToolbarButton (toolbar, "f-spot-rotate-90", new System.EventHandler (HandleRotate90Command));
-		toolbar.AppendSpace ();
-		
-		// FIXME putting these two toggle buttons in a radio group would prevent
-		// the two toggle sounds from being emitted every time you switch modes
-		browse_button = GtkUtil.MakeToolbarToggleButton (toolbar, "f-spot-browse", 
-								 new System.EventHandler (HandleToggleViewBrowse)) as ToggleButton;
-		SetTip (browse_button, Catalog.GetString ("Browse many photos simultaneously"));
-		edit_button = GtkUtil.MakeToolbarToggleButton (toolbar, "f-spot-edit-image", 
-							       new System.EventHandler (HandleToggleViewPhoto)) as ToggleButton;
-		SetTip (edit_button, Catalog.GetString ("View and edit a photo"));
 
-		toolbar.AppendSpace ();
+		ToolButton import_button = GtkUtil.ToolButtonFromTheme ("gtk-add", Catalog.GetString ("Import"), false);
+		import_button.Clicked += HandleImportCommand;
+		import_button.SetTooltip (ToolTips, Catalog.GetString ("Import new images"), null);
+		toolbar.Insert (import_button, -1);
+	
+		toolbar.Insert (new SeparatorToolItem (), -1);
 
-		Widget fs_button = GtkUtil.MakeToolbarButton (toolbar, "f-spot-fullscreen", new System.EventHandler (HandleViewFullscreen));
-		SetTip (fs_button, Catalog.GetString ("View photos fullscreen"));
+		ToolButton rl_button = GtkUtil.ToolButtonFromTheme ("object-rotate-left", Catalog.GetString ("Rotate Left"), true);
+		rl_button.Clicked += HandleRotate270Command;
+		toolbar.Insert (rl_button, -1);
+
+		ToolButton rr_button = GtkUtil.ToolButtonFromTheme ("object-rotate-right", Catalog.GetString ("Rotate Right"), true);
+		rr_button.Clicked += HandleRotate90Command;
+		toolbar.Insert (rr_button, -1);
+
+		toolbar.Insert (new SeparatorToolItem (), -1);
+
+		browse_button = new ToggleToolButton ();
+		browse_button.Label = Catalog.GetString ("Browse");
+		browse_button.IconName = "mode-browse";
+		browse_button.IsImportant = true;
+		browse_button.Toggled += HandleToggleViewBrowse;
+		browse_button.SetTooltip (ToolTips, Catalog.GetString ("Browse many photos simultaneously"), null);
+		toolbar.Insert (browse_button, -1);
+
+		edit_button = new ToggleToolButton ();
+		edit_button.Label = Catalog.GetString ("Edit Image");
+		edit_button.IconName = "mode-image-edit";
+		edit_button.IsImportant = true;
+		edit_button.Toggled += HandleToggleViewPhoto;
+		edit_button.SetTooltip (ToolTips, Catalog.GetString ("View and edit a photo"), null);
+		toolbar.Insert (edit_button, -1);
+
+		toolbar.Insert (new SeparatorToolItem (), -1);
+
+		ToolButton fs_button = GtkUtil.ToolButtonFromTheme ("view-fullscreen", Catalog.GetString ("Fullscreen"), true);
+		fs_button.Clicked += HandleViewFullscreen;
+		fs_button.SetTooltip (ToolTips, Catalog.GetString ("View photos fullscreen"), null);
+		toolbar.Insert (fs_button, -1);
+
+		ToolButton ss_button = GtkUtil.ToolButtonFromTheme ("media-playback-start", Catalog.GetString ("Slideshow"), true);
+		ss_button.Clicked += HandleViewSlideShow;
+		ss_button.SetTooltip (ToolTips, Catalog.GetString ("View photos in a slideshow"), null);
+		toolbar.Insert (ss_button, -1);
 		
-		Widget ss_button = GtkUtil.MakeToolbarButton (toolbar, "f-spot-slideshow", new System.EventHandler (HandleViewSlideShow));
-		SetTip (ss_button, Catalog.GetString ("View photos in a slideshow"));
-
 		tag_selection_widget = new TagSelectionWidget (db.Tags);
 		tag_selection_scrolled.Add (tag_selection_widget);
 		
@@ -265,6 +307,8 @@ public class MainWindow {
 		tag_selection_widget.PopupMenu += HandleTagSelectionPopupMenu;
 		tag_selection_widget.RowActivated += HandleTagSelectionRowActivated;
 		
+		LoadPreference (Preferences.TAG_ICON_SIZE);
+
 		info_box = new InfoBox ();
 		info_box.VersionIdChanged += HandleInfoBoxVersionIdChange;
 		left_vbox.PackStart (info_box, false, true, 0);
@@ -304,12 +348,13 @@ public class MainWindow {
 
 		find_bar = new FindBar (query, tag_selection_widget.Model);
 		view_vbox.PackStart (find_bar, false, false, 0);
+		view_vbox.ReorderChild (find_bar, 1);
 		main_window.KeyPressEvent += HandleKeyPressEvent;
 		
 		query_widget = new FSpot.QueryWidget (query, db, tag_selection_widget);
 		query_widget.Logic.Changed += HandleQueryLogicChanged;
 		view_vbox.PackStart (query_widget, false, false, 0);
-		view_vbox.ReorderChild (query_widget, 1);
+		view_vbox.ReorderChild (query_widget, 2);
 
 		icon_view = new QueryView (query);
 		icon_view.ZoomChanged += HandleZoomChanged;
@@ -329,9 +374,6 @@ public class MainWindow {
 		icon_view.DragBegin += HandleIconViewDragBegin;
 		icon_view.DragDataGet += HandleIconViewDragDataGet;
 
-		near_image.SetFromStock ("f-spot-stock_near", IconSize.SmallToolbar);
-		far_image.SetFromStock ("f-spot-stock_far", IconSize.SmallToolbar);
-
 		PhotoTagMenu pmenu = new PhotoTagMenu ();
 		pmenu.TagSelected += HandleRemoveTagMenuSelected;
 		remove_tag.Submenu = pmenu;
@@ -382,6 +424,25 @@ public class MainWindow {
 		this.selection.Changed += HandleSelectionChanged;
 		this.selection.ItemsChanged += HandleSelectionItemsChanged;
 
+		try {
+			if (export.Submenu != null)
+				export.RemoveSubmenu ();
+
+			export.Submenu = (Mono.Addins.AddinManager.GetExtensionNode ("/FSpot/Menus/Exports") as FSpot.Extensions.SubmenuNode).GetMenuItem ().Submenu;
+		} catch {
+			Console.WriteLine ("There's (maybe) something wrong with some of the installed extensions. You can try removing the directory addin-db-000 from ~/.gnome2/f-spot/");
+		}
+
+		try {
+			if (tools.Submenu != null)
+				tools.RemoveSubmenu ();
+
+			tools.Submenu = (Mono.Addins.AddinManager.GetExtensionNode ("/FSpot/Menus/Tools") as FSpot.Extensions.SubmenuNode).GetMenuItem ().Submenu;
+		} catch {
+			Console.WriteLine ("There's (maybe) something wrong with some of the installed extensions. You can try removing the directory addin-db-000 from ~/.gnome2/f-spot/");
+			tools.Visible = false;
+		}
+
 		UpdateMenus ();
 
 		main_window.ShowAll ();
@@ -405,8 +466,10 @@ public class MainWindow {
 		// When the icon_view is loaded, set it's initial scroll position
 		icon_view.SizeAllocated += HandleIconViewReady;
 
+		export.Activated += HandleExportActivated;
 		UpdateToolbar ();
 
+		Banshee.Kernel.Scheduler.Resume ();
 	}
 
 	private Photo CurrentPhoto {
@@ -480,20 +543,22 @@ public class MainWindow {
 		}
 	}
 
+	private void HandleExportActivated (object o, EventArgs e)
+	{
+		FSpot.Extensions.ExportMenuItemNode.SelectedImages = delegate () {return new FSpot.PhotoArray (SelectedPhotos ()); };
+	}
+
 	private void HandleDbItemsChanged (object sender, DbItemEventArgs args)
 	{
 		foreach (DbItem item in args.Items) {
 			Photo p = item as Photo;
 			if (p == null)
 				continue;
-
-			if (write_metadata) {
-				try {
-					p.WriteMetadataToImage ();
-				} catch (System.Exception e) {
-					Console.WriteLine ("Error syncing metadata to file\n{0}", e);
-				}
-			}
+#if ENABLE_BEAGLE
+			BeagleNotifier.SendUpdate (p);
+#endif
+			if (write_metadata)
+				FSpot.Jobs.SyncMetadataJob.Create (db.Jobs, p);
 		}
 		
 		if (args is TimeChangedEventArgs)
@@ -1108,6 +1173,7 @@ public class MainWindow {
 	{
 		ImportCommand command = new ImportCommand (main_window);
 		if (command.ImportFromPaths (db.Photos, list.ToLocalPaths (), copy) > 0) {
+			query.RollSet = new RollSet (db.Rolls.GetRolls (1)[0]);
 			UpdateQuery ();
 		}
 	}
@@ -1116,6 +1182,7 @@ public class MainWindow {
 	{
 		ImportCommand command = new ImportCommand (main_window);
 		if (command.ImportFromFile (db.Photos, path) > 0) {
+			query.RollSet = new RollSet (db.Rolls.GetRolls (1)[0]);
 			UpdateQuery ();
 		}
 	}
@@ -1325,6 +1392,7 @@ public class MainWindow {
 		db.Sync = false;
 		ImportCommand command = new ImportCommand (main_window);
 		if (command.ImportFromFile (db.Photos, null) > 0) {
+			query.RollSet = new RollSet (db.Rolls.GetRolls (1)[0]);
 			UpdateQuery ();
 		}
 		db.Sync = true;		
@@ -1414,6 +1482,11 @@ public class MainWindow {
 	{
 		PreferenceDialog.Show ();
 	}
+
+	public void HandleManageExtensions (object sender, EventArgs args)
+	{
+		Mono.Addins.Gui.AddinManagerWindow.Run (main_window);
+	}
 	
 	void HandleViewFullExif (object sender, EventArgs args)
 	{
@@ -1437,21 +1510,6 @@ public class MainWindow {
 		info_display_window.Destroyed += HandleInfoDisplayDestroy;
 	}
 
-	void HandleExportToPicasa (object sender, EventArgs args)
-	{
-		new FSpot.GoogleExport (new FSpot.PhotoArray (SelectedPhotos ()));
-	}
-
-	void HandleExportToGallery (object sender, EventArgs args)
-	{
-		new FSpot.GalleryExport (new FSpot.PhotoArray (SelectedPhotos ()));
-	}
-
-	void HandleExportToOriginal (object sender, EventArgs args)
-	{
-		new FSpot.FolderExport (new FSpot.PhotoArray (SelectedPhotos ()));
-	}
-
 	void HandleViewDirectory (object sender, EventArgs args)
 	{
 		Gtk.Window win = new Gtk.Window ("Directory View");
@@ -1476,31 +1534,6 @@ public class MainWindow {
 		win.ShowAll ();
 	}
 
-	void HandleExportToFlickr (object sender, EventArgs args)
-	{
-		new FSpot.FlickrExport (FlickrNet.SupportedService.Flickr, new FSpot.PhotoArray (SelectedPhotos ()), true);
-	}
-	
-	void HandleExportToSmugMug (object sender, System.EventArgs args)
-	{
-		new FSpot.SmugMugExport (new PhotoArray (SelectedPhotos ())); 	
-	}
-
-	void HandleExportTo23hq (object sender, EventArgs args)
-	{
-		new FSpot.FlickrExport (FlickrNet.SupportedService.TwentyThreeHQ, new FSpot.PhotoArray (SelectedPhotos ()), true);
-	}
-
-	void HandleExportToFotki (object sender, EventArgs args)
-	{
-		
-	}
-	
-	void HandleExportToCD (object sender, EventArgs args)
-	{
-		new FSpot.CDExport (new FSpot.PhotoArray (SelectedPhotos ()));
-	}
-
 	private void TestDisplay ()
 	{
 		Gtk.Window win = new Gtk.Window ("hello");
@@ -1525,6 +1558,7 @@ public class MainWindow {
 //				"f-spot.xml");
 	}
 
+	private static About about = null;
 	public static void HandleAbout (object sender, EventArgs args)
 	{
 		string [] authors = new string [] {
@@ -1566,15 +1600,47 @@ public class MainWindow {
                 // * which will give them credit in the About box.
                 // * E.g. "Martin Willemoes Hansen"
                 string translators = Catalog.GetString ("translator-credits");
-                if(System.String.Compare(translators,"translator-credits") == 0) {
-                    translators = null;
+                if (System.String.Compare (translators, "translator-credits") == 0) {
+                	translators = null;
                 }
 
-                new About ("F-Spot", 
+		if (about == null)
+			about = new About ("F-Spot", 
 			   FSpot.Defines.VERSION, 
 			   Catalog.GetString ("Copyright \x00a9 2003-2007 Novell Inc."),
                            null, authors, new string [0], translators, 
-			   PixbufUtils.LoadFromAssembly("f-spot-logo-tango.svg")).Show();
+			   PixbufUtils.LoadFromAssembly ("f-spot-logo.svg"));
+		about.Destroyed += delegate (object o, EventArgs e) {about = null;};
+		about.Show ();
+	}
+
+	void HandleTagSizeChange (object sender, EventArgs args)
+	{
+		RadioMenuItem choice = sender as RadioMenuItem;
+	
+		//Get this callback twice. Once for the active going menuitem,
+		//once for the inactive leaving one. Ignore the inactive.
+		if (!choice.Active)
+			return;
+
+		int old_size = TagsIconSize;
+		
+		if (choice == tag_icon_hidden) {
+			TagsIconSize = (int) Tag.IconSize.Hidden;
+		} else if (choice == tag_icon_small) {
+			TagsIconSize = (int) Tag.IconSize.Small;
+		} else if (choice == tag_icon_medium) {
+			TagsIconSize = (int) Tag.IconSize.Medium;
+		} else if (choice == tag_icon_large) {
+			TagsIconSize = (int) Tag.IconSize.Large;
+		} else {
+			return;
+		}
+		
+		if (old_size != TagsIconSize) {
+			tag_selection_widget.ColumnsAutosize();
+			Preferences.Set (Preferences.TAG_ICON_SIZE, TagsIconSize);
+		}
 	}
 
 	public void HandleArrangeByTime (object sender, EventArgs args)
@@ -1677,7 +1743,7 @@ public class MainWindow {
 		
 		Preferences.Set (Preferences.SIDEBAR_POSITION,		main_hpaned.Position);
 		Preferences.Set (Preferences.ZOOM,			icon_view.Zoom);
-		
+
 		tag_selection_widget.SaveExpandDefaults ();
 
 		this.Window.Destroy ();
@@ -1982,41 +2048,21 @@ public class MainWindow {
 
 	void HandleViewSlideShow (object sender, EventArgs args)
 	{
-		main_window.GdkWindow.Cursor = new Gdk.Cursor (Gdk.CursorType.Watch);
-		slide_delay.Start ();
-	}
-
-	private bool SlideShow ()
-	{
-		int [] ids = SelectedIds ();
-		Photo [] photos = null;
-		if (ids.Length < 2) {
-			int i = 0;
-			if (ids.Length > 0)
-				i = ids [0];
-
-			// FIXME this should be an  IBrowsableCollection.
-			photos = new Photo [query.Photos.Length];
-			Array.Copy (query.Photos, i, photos, 0, query.Photos.Length - i);
-			Array.Copy (query.Photos, 0, photos, query.Photos.Length - i, i);
-			System.Console.WriteLine (photos.Length);
+		int active = Math.Max (ActiveIndex (), 0);
+		if (fsview == null) {
+			fsview = new FSpot.FullScreenView (query);
+			fsview.Destroyed += HandleFullScreenViewDestroy;
+			fsview.View.Item.Index = active;
+			ColorDialog.SwitchViews (fsview.View);
 		} else {
-			photos = SelectedPhotos ();
+			// FIXME this needs to be another mode like PhotoView and IconView mode.
+			fsview.View.Item.Index = active;
 		}
-
-		if (photos.Length == 0) {
-			Console.WriteLine ("No photos available -- no slideshow");
-			main_window.GdkWindow.Cursor = null;
-			return false;
-		}
-
-		FSpot.FullSlide full = new FSpot.FullSlide (main_window, photos);
-		full.Play ();
-		main_window.GdkWindow.Cursor = null;
-		return false;
+		
+		fsview.Show ();
+		fsview.PlayPause ();
 	}
 
-
 	void HandleToggleViewBrowse (object sender, EventArgs args)
 	{
 		if (view_mode == ModeType.IconView)
@@ -2111,9 +2157,9 @@ public class MainWindow {
 										  query.Count), 
 							 query.Count));
 
-			if (PhotoStore.TotalPhotos != query.Count)
-				label_str.Append (String.Format (Catalog.GetString (" out of {0}"), 
-								 PhotoStore.TotalPhotos));
+			int total_photos = Database.Photos.TotalPhotos;
+			if (total_photos != query.Count)
+				label_str.Append (String.Format (Catalog.GetString (" out of {0}"), total_photos));
 
 			if ((selection != null) && (selection.Count > 0))
 				label_str.Append (String.Format (Catalog.GetPluralString (" ({0} selected)", 
@@ -2209,12 +2255,12 @@ public class MainWindow {
 
 		if (e is UnauthorizedAccessException)
 			msg = String.Format (
-				Catalog.GetString ("No permission to delete the file:\n{0}"), 
-				fname).Replace ("_", "__");
+				Catalog.GetString ("No permission to delete the file:{1}{0}"), 
+				fname, Environment.NewLine).Replace ("_", "__");
 		else
 			msg = String.Format (
-				Catalog.GetString ("An error of type {0} occurred while deleting the file:\n{1}"),
-				e.GetType (), fname.Replace ("_", "__"));
+				Catalog.GetString ("An error of type {0} occurred while deleting the file:{2}{1}"),
+				e.GetType (), fname.Replace ("_", "__"), Environment.NewLine);
 		
 		HigMessageDialog.RunHigConfirmation (
 			main_window, DialogFlags.DestroyWithParent, MessageType.Error,
@@ -2455,6 +2501,19 @@ public class MainWindow {
 		}
 		query.Range = null;
 	}
+
+	void HandleSelectLastRoll (object sender, EventArgs args) {
+		query.RollSet = new RollSet (db.Rolls.GetRolls (1)[0]);
+	}
+
+	void HandleSelectRolls (object sender, EventArgs args) {
+		new LastRolls (query, db.Rolls, main_window);
+	}
+
+	void HandleClearRollFilter (object sender, EventArgs args) {
+		query.RollSet = null;
+	}
+
 	
 	void HandleFindUntagged (object sender, EventArgs args) {
 		if (query.Untagged == find_untagged.Active)
@@ -2530,9 +2589,7 @@ public class MainWindow {
 			break;
 
 		case Preferences.GROUP_ADAPTOR_ORDER_ASC:
-			if (reverse_order.Active != (bool) val)
-				reverse_order.Active = (bool) val;
-			//group_selector.Adaptor.OrderAscending = (bool) val;
+			group_selector.Adaptor.OrderAscending = (bool) val;
 			break;
 
 		case Preferences.GLASS_POSITION:
@@ -2552,6 +2609,15 @@ public class MainWindow {
 				main_hpaned.Position = (int) val;
 			break;
 
+		case Preferences.TAG_ICON_SIZE:
+			int s = (int) val;
+			tag_icon_hidden.Active = (s == (int) Tag.IconSize.Hidden);
+			tag_icon_small.Active = (s == (int) Tag.IconSize.Small);
+			tag_icon_medium.Active = (s == (int) Tag.IconSize.Medium);
+			tag_icon_large.Active = (s == (int) Tag.IconSize.Large);
+
+			break;
+
 		case Preferences.ZOOM:
 			icon_view.Zoom = (double) val;
 			break;
@@ -2644,12 +2710,9 @@ public class MainWindow {
 			find_add_tag_with.Submenu.Dispose ();
 		
 		Gtk.Menu submenu = FSpot.Query.TermMenuItem.GetSubmenu (tag_selection_widget.TagHighlight);
-		if (submenu == null)
-			find_add_tag_with.RemoveSubmenu();
-		else
-			find_add_tag_with.Submenu = submenu;
-		
 		find_add_tag_with.Sensitive = (submenu != null);
+		if (submenu != null) 
+			find_add_tag_with.Submenu = submenu;	
 	}
 	
 	public void HandleAddTagToTerm (object sender, EventArgs args)
@@ -2667,7 +2730,7 @@ public class MainWindow {
 		// account for All and separator menu items
 		item_pos -= 2;
 		
-		FSpot.Query.LogicTerm parent_term = (FSpot.Query.LogicTerm) FSpot.Query.LogicWidget.Root.SubTerms [item_pos];
+		FSpot.Query.Term parent_term = (FSpot.Query.Term) FSpot.Query.LogicWidget.Root.SubTerms [item_pos];
 		
 		if (FSpot.Query.LogicWidget.Box != null) {
 			FSpot.Query.Literal after = parent_term.Last as FSpot.Query.Literal;
@@ -2736,43 +2799,54 @@ public class MainWindow {
 		
 		send_mail.Sensitive = active_selection;
 		print.Sensitive = active_selection;
-		export.Sensitive = active_selection;
 		select_none.Sensitive = active_selection;
 		copy_location.Sensitive = active_selection;
 		exif_data.Sensitive = active_selection;
 		sharpen.Sensitive = active_selection;
 		remove_from_catalog.Sensitive = active_selection;
-
+		
+		last_roll.Sensitive = (db.Rolls.GetRolls (1).Length > 0);
+		select_rolls.Sensitive = (db.Rolls.GetRolls (2).Length > 1);
+		clear_roll_filter.Sensitive = (query.RollSet != null);
+		
 		delete_selected_tag.Sensitive = tag_sensitive;
 		edit_selected_tag.Sensitive = tag_sensitive;
 
 
 		attach_tag_to_selection.Sensitive = tag_sensitive && active_selection;
 		remove_tag_from_selection.Sensitive = tag_sensitive && active_selection;
-		
+	
+		export.Sensitive = active_selection;
+
+		try {
+			tools.Visible = (tools.Submenu as Menu).Children.Length > 0;
+		} catch {
+			tools.Visible = false;
+		}
+
 		if (rl_button != null) {
 			if (selection.Count == 0) {
 				rl_button.Sensitive = false;
-				SetTip (rl_button, "");
+				rl_button.SetTooltip (ToolTips, Catalog.GetString (String.Empty), null);
 			} else {
 				rl_button.Sensitive = true;
 
 				string msg = Catalog.GetPluralString ("Rotate selected photo left",
 								      "Rotate selected photos left", selection.Count);
-				SetTip (rl_button, String.Format (msg, selection.Count));
+				rl_button.SetTooltip (ToolTips, String.Format (msg, selection.Count), null);
 			}
 		}
 		
 		if (rr_button != null) {
 			if (selection.Count == 0) {
 				rr_button.Sensitive = false;
-				SetTip (rr_button, "");
+				rr_button.SetTooltip (ToolTips, Catalog.GetString (String.Empty), null);
 			} else {
 				rr_button.Sensitive = true;
 
 				string msg = Catalog.GetPluralString ("Rotate selected photo right",
-				    "Rotate selected photos right", selection.Count);
-				SetTip (rr_button, String.Format (msg, selection.Count));
+								      "Rotate selected photos right", selection.Count);
+				rr_button.SetTooltip (ToolTips, String.Format (msg, selection.Count), null);
 			}
 		}
 
@@ -2896,8 +2970,6 @@ public class MainWindow {
 
 		tagbar.Show ();
 		UpdateTagEntryFromSelection ();
-		if (tag_entry.Text.Length > 0)
-			tag_entry.Text += ", ";
 		tag_entry.GrabFocus ();
 		tag_entry.SelectRegion (-1, -1);
 	}
@@ -3008,7 +3080,7 @@ public class MainWindow {
 
 	private void ShowQueryWidget () {
 		if (find_bar.Visible) {
-			find_bar.Entry.Text = "";
+			find_bar.Entry.Text = String.Empty;
 			find_bar.Hide ();
 		}
 		
@@ -3026,18 +3098,15 @@ public class MainWindow {
 					query_widget.Close ();
 				}
 				
-				find_bar.Show ();
-				find_bar.Entry.GrabFocus ();
-				args.RetVal = true;
-				return;
+				find_bar.ShowAll();
 			}
+
+			// Grab the focus even if it's already shown
+			find_bar.Entry.GrabFocus ();
+			args.RetVal = true;
+			return;
 		}
 		
 		args.RetVal = false;
 	}
-	
-	public static void SetTip (Widget widget, string tip)
-	{
-		toolTips.SetTip (widget, tip, null);
-	}
 }
