@@ -10,7 +10,9 @@ using System.IO;
 using System.Text;
 using System;
 using FSpot;
+using FSpot.Query;
 
+using Banshee.Database;
 
 public class PhotoVersion : FSpot.IBrowsableItem {
 	Photo photo;
@@ -174,6 +176,9 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		get {
 			return directory_path;
 		}
+		set {
+			directory_path = value;
+		}
 	}
 
 	private ArrayList tags;
@@ -206,6 +211,12 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		}
 	}
 
+	private uint roll_id = 0;
+	public uint RollId {
+		get { return roll_id; }
+		set { roll_id = value; }
+	}
+
 	// Version management
 	public const int OriginalVersionId = 1;
 	private uint highest_version_id;
@@ -510,55 +521,56 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 	{
 		string path = this.DefaultVersionUri.LocalPath;
 
-		FSpot.ImageFile img = FSpot.ImageFile.Create (DefaultVersionUri);
-		if (img is FSpot.JpegFile) {
-			FSpot.JpegFile jimg = img as FSpot.JpegFile;
+		using (FSpot.ImageFile img = FSpot.ImageFile.Create (DefaultVersionUri)) {
+			if (img is FSpot.JpegFile) {
+				FSpot.JpegFile jimg = img as FSpot.JpegFile;
 			
-			jimg.SetDescription (this.Description);
-			jimg.SetDateTimeOriginal (this.Time.ToLocalTime ());
-			jimg.SetXmp (UpdateXmp (this, jimg.Header.GetXmp ()));
+				jimg.SetDescription (this.Description);
+				jimg.SetDateTimeOriginal (this.Time.ToLocalTime ());
+				jimg.SetXmp (UpdateXmp (this, jimg.Header.GetXmp ()));
 
-			jimg.SaveMetaData (path);
-		} else if (img is FSpot.Png.PngFile) {
-			FSpot.Png.PngFile png = img as FSpot.Png.PngFile;
+				jimg.SaveMetaData (path);
+			} else if (img is FSpot.Png.PngFile) {
+				FSpot.Png.PngFile png = img as FSpot.Png.PngFile;
 			
-			if (img.Description != this.Description)
-				png.SetDescription (this.Description);
+				if (img.Description != this.Description)
+					png.SetDescription (this.Description);
 			
-			png.SetXmp (UpdateXmp (this, png.GetXmp ()));
+				png.SetXmp (UpdateXmp (this, png.GetXmp ()));
 
-			png.Save (path);
+				png.Save (path);
+			}
 		}
 	}
 
 	public uint SaveVersion (Gdk.Pixbuf buffer, bool create_version)
 	{
 		uint version = DefaultVersionId;
-		ImageFile img = ImageFile.Create (DefaultVersionUri);
+		using (ImageFile img = ImageFile.Create (DefaultVersionUri)) {
+			// Always create a version if the source is not a jpeg for now.
+			create_version = create_version || !(img is FSpot.JpegFile);
 
-		// Always create a version if the source is not a jpeg for now.
-		create_version = create_version || !(img is FSpot.JpegFile);
+			if (buffer == null)
+				throw new ApplicationException ("invalid (null) image");
 
-		if (buffer == null)
-			throw new ApplicationException ("invalid (null) image");
-
-		if (create_version)
-			version = CreateDefaultModifiedVersion (DefaultVersionId, false);
-
-		try {
-			string version_path = GetVersionPath (version);
-			
-			using (Stream stream = File.OpenWrite (version_path)) {
-				img.Save (buffer, stream);
-			}
-			FSpot.ThumbnailGenerator.Create (version_path).Dispose ();
-			DefaultVersionId = version;
-		} catch (System.Exception e) {
-			System.Console.WriteLine (e);
 			if (create_version)
-				DeleteVersion (version);
+				version = CreateDefaultModifiedVersion (DefaultVersionId, false);
+
+			try {
+				string version_path = GetVersionPath (version);
 			
-			throw e;
+				using (Stream stream = File.OpenWrite (version_path)) {
+					img.Save (buffer, stream);
+				}
+				FSpot.ThumbnailGenerator.Create (version_path).Dispose ();
+				DefaultVersionId = version;
+			} catch (System.Exception e) {
+				System.Console.WriteLine (e);
+				if (create_version)
+					DeleteVersion (version);
+			
+				throw e;
+			}
 		}
 		
 		return version;
@@ -573,7 +585,7 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 		this.directory_path = directory_path;
 		this.name = name;
 
-		description = "";
+		description = String.Empty;
 
 		// Note that the original version is never stored in the photo_versions table in the
 		// database.
@@ -591,13 +603,14 @@ public class Photo : DbItem, IComparable, FSpot.IBrowsableItem {
 }
 
 public class PhotoStore : DbStore {
-	TagStore tag_store;
-
-
-	static int total_photos = 0;
-	public static int TotalPhotos {
-		get { return total_photos; }
-		set { total_photos = value; }
+	public int TotalPhotos {
+		get {
+			SqliteDataReader reader = Database.Query("SELECT COUNT(*) FROM photos");
+			reader.Read ();
+			int total = Convert.ToInt32 (reader [0]);
+			reader.Close ();
+			return total;
+		}
 	}
 
 	public static ThumbnailFactory ThumbnailFactory = new ThumbnailFactory (ThumbnailSize.Large);
@@ -620,9 +633,9 @@ public class PhotoStore : DbStore {
 
 	public static Pixbuf GenerateThumbnail (Uri uri)
 	{
-		FSpot.ImageFile img = FSpot.ImageFile.Create (uri);
-		
-		return GenerateThumbnail (uri, img);
+		using (FSpot.ImageFile img = FSpot.ImageFile.Create (uri)) {
+			return GenerateThumbnail (uri, img);
+		}
 	}
 
 	public static Pixbuf GenerateThumbnail (Uri uri, ImageFile img)
@@ -664,135 +677,102 @@ public class PhotoStore : DbStore {
 
 	// Constructor
 
-	public PhotoStore (SqliteConnection connection, bool is_new, TagStore tag_store)
-		: base (connection, false)
+	public PhotoStore (QueuedSqliteDatabase database, bool is_new)
+		: base (database, false)
 	{
-		this.tag_store = tag_store;
 		EnsureThumbnailDirectory ();
 
-		SqliteCommand command;
-
-		if (! is_new) {
-			command = new SqliteCommand ();
-			command.Connection = Connection;
-			command.CommandText = String.Format ("SELECT count(*) from photos");
-			SqliteDataReader reader = command.ExecuteReader ();
-	
-			while (reader.Read ()) {
-				TotalPhotos = Convert.ToInt32 (reader [0]);
-			}
-			command.Dispose ();
-		
+		if (! is_new)
 			return;
-		}
 		
-		ExecuteSqlCommand ( 
+		Database.ExecuteNonQuery ( 
 			"CREATE TABLE photos (                                     " +
 			"	id                 INTEGER PRIMARY KEY NOT NULL,   " +
 			"       time               INTEGER NOT NULL,	   	   " +
 			"       directory_path     STRING NOT NULL,		   " +
 			"       name               STRING NOT NULL,		   " +
 			"       description        TEXT NOT NULL,	           " +
+			"       roll_id            INTEGER NOT NULL,		   " +
 			"       default_version_id INTEGER NOT NULL		   " +
 			")");
 
 
-		ExecuteSqlCommand (
+		Database.ExecuteNonQuery (
 			"CREATE TABLE photo_tags (     " +
 			"	photo_id      INTEGER, " +
 			"       tag_id        INTEGER  " +
 			")");
 
 
-		ExecuteSqlCommand (
+		Database.ExecuteNonQuery (
 			"CREATE TABLE photo_versions (    " +
 			"       photo_id        INTEGER,  " +
 			"       version_id      INTEGER,  " +
 			"       name            STRING    " +
 			")");
-
-		TotalPhotos = 0;
 	}
 
 
-	public Photo Create (string path, out Pixbuf thumbnail)
+	public Photo Create (string path, uint roll_id, out Pixbuf thumbnail)
 	{
-		return Create (path, path, out thumbnail);
+		return Create (path, path, roll_id, out thumbnail);
+
 	}
 
-	public Photo Create (string newPath, string origPath, out Pixbuf thumbnail)
+	public Photo Create (string newPath, string origPath, uint roll_id, out Pixbuf thumbnail)
 	{
-		FSpot.ImageFile img = FSpot.ImageFile.Create (origPath);
-		long unix_time = DbUtils.UnixTimeFromDateTime (img.Date);
-		string description = img.Description != null  ? img.Description.Split ('\0') [0] : "";
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-
-		command.CommandText = String.Format ("INSERT INTO photos (time, " +
-						     "directory_path, name, description, default_version_id) " +
-						     "       VALUES ({0}, '{1}', '{2}', '{3}', {4})",
-						     unix_time,
-						     SqlString (System.IO.Path.GetDirectoryName (newPath)),
-						     SqlString (System.IO.Path.GetFileName (newPath)),
-						     SqlString (description),
-						     Photo.OriginalVersionId);
-
-		command.ExecuteScalar ();
-		command.Dispose ();
-
-		uint id = (uint) Connection.LastInsertRowId;
-		Photo photo = new Photo (id, unix_time, newPath);
-		AddToCache (photo);
-		photo.Loaded = true;
-
-		thumbnail = GenerateThumbnail (UriList.PathToFileUri (newPath), img);		
-		EmitAdded (photo);
-
-		TotalPhotos++;
-		
+		Photo photo;
+		using (FSpot.ImageFile img = FSpot.ImageFile.Create (origPath)) {
+			long unix_time = DbUtils.UnixTimeFromDateTime (img.Date);
+			string description = img.Description != null  ? img.Description.Split ('\0') [0] : String.Empty;
+	
+	 		uint id = (uint) Database.Execute (new DbCommand ("INSERT INTO photos (time, "	+
+					"directory_path, name, description, roll_id, default_version_id) "	+
+	 				"VALUES (:time, :directory_path, :name, :description, "		+
+					":roll_id, :default_version_id)",
+	 				"time", unix_time,
+	 				"directory_path", System.IO.Path.GetDirectoryName (newPath),
+	 				"name", System.IO.Path.GetFileName (newPath),
+	 				"description", description,
+					"roll_id", roll_id,
+	 				"default_version_id", Photo.OriginalVersionId));
+	
+			photo = new Photo (id, unix_time, newPath);
+			AddToCache (photo);
+			photo.Loaded = true;
+	
+			thumbnail = GenerateThumbnail (UriList.PathToFileUri (newPath), img);		
+			EmitAdded (photo);
+		}
 		return photo;
 	}
 
 	private void GetVersions (Photo photo)
 	{
-
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-		command.CommandText = String.Format ("SELECT version_id, name FROM photo_versions WHERE photo_id = {0}", photo.Id);
-		SqliteDataReader reader = command.ExecuteReader ();
+		SqliteDataReader reader = Database.Query(new DbCommand("SELECT version_id, name FROM photo_versions WHERE photo_id = :id", photo.Id));
 
 		while (reader.Read ()) {
 			uint version_id = Convert.ToUInt32 (reader [0]);
 			string name = reader[1].ToString ();
 			photo.AddVersionUnsafely (version_id, name);
 		}
-
-		command.Dispose ();
+		reader.Close();
 	}
 
 	private void GetTags (Photo photo)
 	{
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-		command.CommandText = String.Format ("SELECT tag_id FROM photo_tags WHERE photo_id = {0}", photo.Id);
-		SqliteDataReader reader = command.ExecuteReader ();
+		SqliteDataReader reader = Database.Query(new DbCommand("SELECT tag_id FROM photo_tags WHERE photo_id = :id", photo.Id));
 
 		while (reader.Read ()) {
 			uint tag_id = Convert.ToUInt32 (reader [0]);
-			Tag tag = tag_store.Get (tag_id) as Tag;
+			Tag tag = Core.Database.Tags.Get (tag_id) as Tag;
 			photo.AddTagUnsafely (tag);
 		}
-
-		command.Dispose ();
+		reader.Close();
 	}		
 	
 	private void GetAllVersions  () {
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-		command.CommandText = String.Format ("SELECT photo_id, version_id, name " +
-						     "FROM photo_versions ");
-		
-		SqliteDataReader reader = command.ExecuteReader ();
+		SqliteDataReader reader = Database.Query("SELECT photo_id, version_id, name FROM photo_versions");
 		
 		while (reader.Read ()) {
 			uint id = Convert.ToUInt32 (reader [0]);
@@ -821,18 +801,12 @@ public class PhotoStore : DbStore {
 				directory_path = reader [3].ToString ();
 			System.Console.WriteLine ("directory_path = {0}", directory_path);
 			*/
-
-
 		}
+		reader.Close();
 	}
 
 	private void GetAllTags () {
-			SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-		command.CommandText = String.Format ("SELECT photo_id, tag_id " +
-						     "FROM photo_tags ");
-		
-		SqliteDataReader reader = command.ExecuteReader ();
+		SqliteDataReader reader = Database.Query("SELECT photo_id, tag_id FROM photo_tags");
 
 		while (reader.Read ()) {
 			uint id = Convert.ToUInt32 (reader [0]);
@@ -850,20 +824,17 @@ public class PhotoStore : DbStore {
 
 		        if (reader [1] != null) {
 				uint tag_id = Convert.ToUInt32 (reader [1]);
-				Tag tag = tag_store.Get (tag_id) as Tag;
+				Tag tag = Core.Database.Tags.Get (tag_id) as Tag;
 				photo.AddTagUnsafely (tag);
 			}
 		}
+		reader.Close();
 	}
 
 	private void GetAllData () {
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-		command.CommandText = String.Format ("SELECT photo_tags.photo_id, tag_id, version_id, name " +
-						     "FROM photo_tags, photo_versions " +
-						     "WHERE photo_tags.photo_id = photo_versions.photo_id");
-		
-		SqliteDataReader reader = command.ExecuteReader ();
+		SqliteDataReader reader = Database.Query("SELECT photo_tags.photo_id, tag_id, version_id, name "
+                                               + "FROM photo_tags, photo_versions "
+                                               + "WHERE photo_tags.photo_id = photo_versions.photo_id");
 
 		while (reader.Read ()) {
 			uint id = Convert.ToUInt32 (reader [0]);
@@ -881,7 +852,7 @@ public class PhotoStore : DbStore {
 
 		        if (reader [1] != null) {
 				uint tag_id = Convert.ToUInt32 (reader [1]);
-				Tag tag = tag_store.Get (tag_id) as Tag;
+				Tag tag = Core.Database.Tags.Get (tag_id) as Tag;
 				photo.AddTagUnsafely (tag);
 			}
 			if (reader [2] != null) {
@@ -891,23 +862,20 @@ public class PhotoStore : DbStore {
 				photo.AddVersionUnsafely (version_id, name);
 			}
 		}
+		reader.Close();
 	}
 
 	private void GetData (Photo photo)
 	{
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-		command.CommandText = String.Format ("SELECT tag_id, version_id, name " +
-						     "  FROM photo_tags, photo_versions " +
-						     " WHERE photo_tags.photo_id = photo_versions.photo_id " +
-						     "   AND photo_tags.photo_id = {0}", photo.Id);
-
-		SqliteDataReader reader = command.ExecuteReader ();
+		SqliteDataReader reader = Database.Query(new DbCommand("SELECT tag_id, version_id, name "
+                                                             + "FROM photo_tags, photo_versions "
+                                                             + "WHERE photo_tags.photo_id = photo_versions.photo_id "
+                                                             + "AND photo_tags.photo_id = :id", "id", photo.Id));
 
 		while (reader.Read ()) {
 		        if (reader [0] != null) {
 				uint tag_id = Convert.ToUInt32 (reader [0]);
-				Tag tag = tag_store.Get (tag_id) as Tag;
+				Tag tag = Core.Database.Tags.Get (tag_id) as Tag;
 				photo.AddTagUnsafely (tag);
 			}
 			if (reader [1] != null) {
@@ -917,6 +885,7 @@ public class PhotoStore : DbStore {
 				photo.AddVersionUnsafely (version_id, name);
 			}
 		}
+		reader.Close();
 	}
 
 	public override DbItem Get (uint id)
@@ -925,17 +894,8 @@ public class PhotoStore : DbStore {
 		if (photo != null)
 			return photo;
 
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-		command.CommandText = String.Format ("SELECT time,                                 " +
-						     "       directory_path,                       " +
-						     "       name,                                 " +
-						     "       description,                          " +
-						     "       default_version_id                    " +
-						     "     FROM photos                             " +
-						     "     WHERE id = {0}                          ",
-						     id);
-		SqliteDataReader reader = command.ExecuteReader ();
+		SqliteDataReader reader = Database.Query(new DbCommand("SELECT time, directory_path, name, description, "
+                                                     + "roll_id, default_version_id FROM photos WHERE id = :id", "id", id));
 
 		if (reader.Read ()) {
 			photo = new Photo (id,
@@ -944,11 +904,11 @@ public class PhotoStore : DbStore {
 					   reader [2].ToString ());
 
 			photo.Description = reader[3].ToString ();
-			photo.DefaultVersionId = Convert.ToUInt32 (reader[4]);
+			photo.RollId = Convert.ToUInt32 (reader[4]);
+			photo.DefaultVersionId = Convert.ToUInt32 (reader[5]);
 			AddToCache (photo);
 		}
-
-		command.Dispose ();
+		reader.Close();
 
 		if (photo == null)
 			return null;
@@ -965,23 +925,12 @@ public class PhotoStore : DbStore {
 		//        this is only used for DND
 
 		Photo photo = null;
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
 
 		string directory_path = System.IO.Path.GetDirectoryName (path);
 		string filename = System.IO.Path.GetFileName (path);
 
-		command.CommandText = String.Format ("SELECT id,                                   " +
-				                     "       time,                                 " +
-						     "       description,                          " +
-						     "       default_version_id                    " +
-						     "  FROM photos                                " +
-						     " WHERE directory_path = \"{0}\"              " +
-						     "   AND name = \"{1}\"                        ",
-						     directory_path,
-						     filename);
-
-		SqliteDataReader reader = command.ExecuteReader ();
+		SqliteDataReader reader = Database.Query(new DbCommand("SELECT id, time, description, roll_id, default_version_id FROM photos "
+                + "WHERE directory_path = :directory_path AND name = :name", "directory_path", directory_path, "name", filename));
 
 		if (reader.Read ()) {
 			photo = new Photo (Convert.ToUInt32 (reader [0]),
@@ -990,11 +939,11 @@ public class PhotoStore : DbStore {
 					   filename);
 
 			photo.Description = reader[2].ToString ();
-			photo.DefaultVersionId = Convert.ToUInt32 (reader[3]);
+			photo.RollId = Convert.ToUInt32 (reader[3]);
+			photo.DefaultVersionId = Convert.ToUInt32 (reader[4]);
 			AddToCache (photo);
 		}
-
-		command.Dispose ();
+        reader.Close();
 
 		if (photo == null)
 			return null;
@@ -1015,7 +964,7 @@ public class PhotoStore : DbStore {
 		}
 		
 		foreach (Tag tag in tags)
-			tag_store.Remove (tag);
+			Core.Database.Tags.Remove (tag);
 		
 	}
 
@@ -1034,14 +983,11 @@ public class PhotoStore : DbStore {
 			query_builder.Append (String.Format ("id = {0}", items[i].Id));
 			tv_query_builder.Append (String.Format ("photo_id = {0}", items[i].Id));
 			RemoveFromCache (items[i]);
-			TotalPhotos--;			
 		}
 
-		ExecuteSqlCommand (String.Format ("DELETE FROM photos WHERE {0}", query_builder.ToString ()));
-//
-		ExecuteSqlCommand (String.Format ("DELETE FROM photo_tags WHERE {0}", tv_query_builder.ToString ()));
-//
-		ExecuteSqlCommand (String.Format ("DELETE FROM photo_versions WHERE {0}", tv_query_builder.ToString ()));
+		Database.ExecuteNonQuery (String.Format ("DELETE FROM photos WHERE {0}", query_builder.ToString ()));
+		Database.ExecuteNonQuery (String.Format ("DELETE FROM photo_tags WHERE {0}", tv_query_builder.ToString ()));
+		Database.ExecuteNonQuery (String.Format ("DELETE FROM photo_versions WHERE {0}", tv_query_builder.ToString ()));
 
 	}
 
@@ -1059,7 +1005,7 @@ public class PhotoStore : DbStore {
 	public void Commit (DbItem [] items, DbItemEventArgs args)
 	{
 		if (items.Length > 1)
-			BeginTransaction ();
+			Database.BeginTransaction ();
 
 		foreach (DbItem item in items) {
 			Update ((Photo)item);
@@ -1067,34 +1013,34 @@ public class PhotoStore : DbStore {
 		EmitChanged (items, args);
 
 		if (items.Length > 1)
-			CommitTransaction ();
+			Database.CommitTransaction ();
 	}
 	
-        private void Update (Photo photo) {
+	private void Update (Photo photo) {
 		// Update photo.
 
-		ExecuteSqlCommand (String.Format ("UPDATE photos SET description = '{0}',     " +
-						     "                  default_version_id = {1}, " +
-						     "                  time = {2} " +
-						     "              WHERE id = {3}",
-						     SqlString (photo.Description),
-						     photo.DefaultVersionId,
-						     DbUtils.UnixTimeFromDateTime (photo.Time),
-						     photo.Id));
+		Database.ExecuteNonQuery (new DbCommand ("UPDATE photos SET description = :description, " + 
+						     "directory_path = :directory_path, " +
+						     "default_version_id = :default_version_id, time = :time WHERE id = :id ",
+						     "description", photo.Description,
+						     "directory_path", photo.DirectoryPath,
+						     "default_version_id", photo.DefaultVersionId,
+						     "time", DbUtils.UnixTimeFromDateTime (photo.Time),
+						     "id", photo.Id));
 
 		// Update tags.
 
-		ExecuteSqlCommand (String.Format ("DELETE FROM photo_tags WHERE photo_id = {0}", photo.Id));
+		Database.ExecuteNonQuery (new DbCommand ("DELETE FROM photo_tags WHERE photo_id = :id", "id", photo.Id));
 
 		foreach (Tag tag in photo.Tags) {
-			ExecuteSqlCommand (String.Format ("INSERT INTO photo_tags (photo_id, tag_id) " +
-							     "       VALUES ({0}, {1})",
-							     photo.Id, tag.Id));
+			Database.ExecuteNonQuery (new DbCommand ("INSERT INTO photo_tags (photo_id, tag_id) " +
+                                         " VALUES (:photo_id, :tag_id)",
+                                         "photo_id", photo.Id, "tag_id", tag.Id));
 		}
 
 		// Update versions.
 
-		ExecuteSqlCommand (String.Format ("DELETE FROM photo_versions WHERE photo_id = {0}", photo.Id));
+		Database.ExecuteNonQuery (new DbCommand ("DELETE FROM photo_versions WHERE photo_id = :id", "id", photo.Id));
 
 		foreach (uint version_id in photo.VersionIds) {
 			if (version_id == Photo.OriginalVersionId)
@@ -1102,9 +1048,9 @@ public class PhotoStore : DbStore {
 
 			string version_name = photo.GetVersionName (version_id);
 
-			ExecuteSqlCommand (String.Format ("INSERT INTO photo_versions (photo_id, version_id, name) " +
-							     "       VALUES ({0}, {1}, '{2}')",
-							     photo.Id, version_id, SqlString (version_name)));
+			Database.ExecuteNonQuery(new DbCommand ("INSERT INTO photo_versions (photo_id, version_id, name) " +
+							     "       VALUES (:photo_id, :version_id, :name)",
+							     "photo_id", photo.Id, "version_id", version_id, "name", version_name));
 		}
 	}
 	
@@ -1131,23 +1077,43 @@ public class PhotoStore : DbStore {
 		}
 	}
 
+
 	// Queries.
+
+	[Obsolete ("drop this, use IQueryCondition correctly instead")]
+	private string AddLastImportFilter (RollSet roll_set, bool added_where)
+	{
+		if (roll_set == null)
+			return null;
+
+		return  String.Format (" {0}{1}", added_where ? " AND " : " WHERE ", roll_set.SqlClause () );
+	}
+
+	
+	
+	public Photo [] Query (Tag [] tags, DateTime start, DateTime end, Roll [] rolls)
+	{
+		return Query (tags, null, new DateRange (start, end), new RollSet (rolls));
+	}
+
+	public Photo [] Query (Tag [] tags, Roll [] rolls)
+	{
+		return Query (tags, null, null, rolls == null ? null : new RollSet (rolls));
+	}
+
 	public Photo [] Query (Tag [] tags, DateTime start, DateTime end)
 	{
-		return Query (tags, null, new DateRange (start, end));
+		return Query (tags, null, new DateRange (start, end), null);
 	}
 	
 	public Photo [] Query (Tag [] tags) {
-		return Query (tags, null, null);
+		return Query (tags, null, null, null);
 	}
 
 	public Photo [] Query (string query)
 	{
-		SqliteCommand command = new SqliteCommand ();
-		command.Connection = Connection;
-		command.CommandText = query;
-		SqliteDataReader reader = command.ExecuteReader ();
-		
+		SqliteDataReader reader = Database.Query(query);
+
 		ArrayList version_list = new ArrayList ();
 		ArrayList id_list = new ArrayList ();
 		while (reader.Read ()) {
@@ -1161,13 +1127,15 @@ public class PhotoStore : DbStore {
 						   reader [3].ToString ());
 				
 				photo.Description = reader[4].ToString ();
-				photo.DefaultVersionId = Convert.ToUInt32 (reader[5]);		 
+				photo.RollId = Convert.ToUInt32 (reader[5]);
+				photo.DefaultVersionId = Convert.ToUInt32 (reader[6]);
 				
 				version_list.Add (photo);
 			}
 
 			id_list.Add (photo);
 		}
+		reader.Close();
 
 		bool need_load = false;
 		foreach (Photo photo in version_list) {
@@ -1185,8 +1153,6 @@ public class PhotoStore : DbStore {
 			//Console.WriteLine ("Skipped Loading Data");
 		}
 
-		command.Dispose ();
-
 		return id_list.ToArray (typeof (Photo)) as Photo [];
 	}
 
@@ -1197,6 +1163,7 @@ public class PhotoStore : DbStore {
 						     "       photos.directory_path,              " +
 						     "       photos.name,                        " +
 						     "       photos.description,                 " +
+						     "       photos.roll_id,                     " +
 						     "       photos.default_version_id           " +
 						     "     FROM photos                           " +
 						     "     WHERE directory_path = \"{0}\"", dir.FullName);
@@ -1204,28 +1171,38 @@ public class PhotoStore : DbStore {
 		return Query (query_string);
 	}
 
-	public Photo [] QueryUntagged (DateRange range)
+	public Photo [] QueryUntagged (DateRange range, RollSet importidrange)
 	{
 		StringBuilder query_builder = new StringBuilder ();
 
 		query_builder.Append ("SELECT * FROM photos WHERE id NOT IN " +
 					"(SELECT DISTINCT photo_id FROM photo_tags) ");
 		
+		bool added_where = true;
 		if (range != null) {
 			query_builder.Append (String.Format ("AND photos.time >= {0} AND photos.time <= {1} ",
 							     DbUtils.UnixTimeFromDateTime (range.Start), 
 							     DbUtils.UnixTimeFromDateTime (range.End)));
+			added_where = true;
 		}
 
+		if (importidrange != null) {
+			query_builder.Append (AddLastImportFilter (importidrange, added_where));
+			added_where = true;
+ 		}
+
 		query_builder.Append("ORDER BY time");
 
 		return Query (query_builder.ToString ());
 	}
 
-	public Photo [] Query (Tag [] tags, string extra_condition, DateRange range)
+	public Photo [] Query (Tag [] tags, string extra_condition, DateRange range, RollSet importidrange)
 	{
-		string query;
+		return Query (OrTerm.FromTags(tags), extra_condition, range, importidrange);
+	}
 
+	public Photo [] Query (Term searchexpression, string extra_condition, DateRange range, RollSet importidrange)
+	{
 		bool hide = (extra_condition == null);
 
 		// The SQL query that we want to construct is:
@@ -1235,6 +1212,7 @@ public class PhotoStore : DbStore {
 		//        photos.directory_path,
 		//        photos.name,
 		//        photos.description,
+		//	  photos.roll_id,
 		//        photos.default_version_id
 		//                  FROM photos, photo_tags
 		//                  WHERE photos.id = photo_tags.photo_id
@@ -1242,65 +1220,57 @@ public class PhotoStore : DbStore {
 		//			            OR photo_tags.tag_id = cat1tag2 ) 
 		// 		                AND (photo_tags.tag_id = cat2tag1
 		//			            OR photo_tags.tag_id = cat2tag2 )
+		//			  	AND (photos.roll_id = roll_id1
+		//			   	    OR photos.roll_id = roll_id2 ...)
 		//                  GROUP BY photos.id
 		
 		StringBuilder query_builder = new StringBuilder ();
-		query_builder.Append ("SELECT photos.id,                          " +
-				      "       photos.time,                        " +
-				      "       photos.directory_path,              " +
-				      "       photos.name,                        " +
-				      "       photos.description,                 " +
-				      "       photos.default_version_id           " +
-				      "     FROM photos                      ");
+		query_builder.Append ("SELECT photos.id, " 			+
+					     "photos.time, "			+
+					     "photos.directory_path, " 		+
+					     "photos.name, "			+
+					     "photos.description, "		+
+				      	     "photos.roll_id, "   		+
+					     "photos.default_version_id "	+
+				      "FROM photos ");
 		
+		bool where_statement_added = false;
+
 		if (range != null) {
 			query_builder.Append (String.Format ("WHERE photos.time >= {0} AND photos.time <= {1} ",
 							     DbUtils.UnixTimeFromDateTime (range.Start), 
 							     DbUtils.UnixTimeFromDateTime (range.End)));
-		}
-		
-		if (hide && tag_store.Hidden != null) {
-			query_builder.Append (String.Format ("{0} photos.id NOT IN (SELECT photo_id FROM photo_tags WHERE tag_id = {1})", 
-							     range != null ? " AND " : " WHERE ", tag_store.Hidden.Id));
-		}
-		
-		if (tags != null && tags.Length > 0) {
-			bool first = true;
-			foreach (Tag t in tags) {
-				if (t == null)
-					continue;
-				
-				if (tag_store.Hidden != null && t.Id == tag_store.Hidden.Id)
-					continue;
-				
-				if (first) {
-					query_builder.Append (String.Format ("{0} photos.id IN (SELECT photo_id FROM photo_tags WHERE tag_id IN (",
-									     (hide && tag_store.Hidden != null) || range != null ? " AND " : " WHERE "));
-				}
-				
-				query_builder.Append (String.Format ("{0}{1} ", first ? "" : ", ", t.Id));
-				
-					first = false;
-			}
-			
-			if (!first)
-				query_builder.Append (")) ");
+			where_statement_added = true;
 		}
 
-		if (extra_condition != null) {
-			query_builder.Append (
-				String.Format (
-					"{0} {1} ",
-					(hide || range != null || (tags != null && tags.Length > 0)) ? " AND " : " WHERE ",
-					extra_condition
-				)
-			);
+		if (importidrange != null) {
+			query_builder.Append (AddLastImportFilter (importidrange, where_statement_added));
+			where_statement_added = true;
+		}		
+		
+		if (hide && Core.Database.Tags.Hidden != null) {
+			query_builder.Append (String.Format ("{0} photos.id NOT IN (SELECT photo_id FROM photo_tags WHERE tag_id = {1}) ", 
+							     where_statement_added ? " AND " : " WHERE ", Core.Database.Tags.Hidden.Id));
+			where_statement_added = true;
+		}
+		
+		if (searchexpression != null) {
+			query_builder.Append (String.Format ("{0} {1}", 
+							     where_statement_added ? " AND " : " WHERE ",
+							     searchexpression.SqlCondition()));
+			where_statement_added = true;
+		}
+
+		if (extra_condition != null && extra_condition.Length != 0) {
+			query_builder.Append (String.Format ("{0} {1} ",
+							     where_statement_added ? " AND " : " WHERE ",
+							     extra_condition));
+			where_statement_added = true;
 		}
 		
 		query_builder.Append ("ORDER BY photos.time");
-		query = query_builder.ToString ();
-		
-		return Query (query);
+		Console.WriteLine("Query: {0}", query_builder.ToString());
+		return Query (query_builder.ToString ());
 	}
 
 #if TEST_PHOTO_STORE
@@ -1309,7 +1279,7 @@ public class PhotoStore : DbStore {
 		Console.WriteLine ("\t[{0}] {1}", photo.Id, photo.Path);
 		Console.WriteLine ("\t{0}", photo.Time.ToLocalTime ());
 
-		if (photo.Description != "")
+		if (photo.Description != String.Empty)
 			Console.WriteLine ("\t{0}", photo.Description);
 		else
 			Console.WriteLine ("\t(no description)");
@@ -1373,14 +1343,14 @@ public class PhotoStore : DbStore {
 
 		Pixbuf unused_thumbnail;
 
-		Photo ny_landscape = db.Photos.Create (DateTime.Now.ToUniversalTime (), "/home/ettore/Photos/ny_landscape.jpg",
+		Photo ny_landscape = db.Photos.Create (DateTime.Now.ToUniversalTime (), 1, "/home/ettore/Photos/ny_landscape.jpg",
 						       out unused_thumbnail);
 		ny_landscape.Description = "Pretty NY skyline";
 		ny_landscape.AddTag (landscapes_tag);
 		ny_landscape.AddTag (favorites_tag);
 		db.Photos.Commit (ny_landscape);
 
-		Photo me_in_sf = db.Photos.Create (DateTime.Now.ToUniversalTime (), "/home/ettore/Photos/me_in_sf.jpg",
+		Photo me_in_sf = db.Photos.Create (DateTime.Now.ToUniversalTime (), 2, "/home/ettore/Photos/me_in_sf.jpg",
 						   out unused_thumbnail);
 		me_in_sf.AddTag (landscapes_tag);
 		me_in_sf.AddTag (portraits_tag);
@@ -1393,7 +1363,7 @@ public class PhotoStore : DbStore {
 		me_in_sf.CreateVersion ("UM-ed", Photo.OriginalVersionId);
 		db.Photos.Commit (me_in_sf);
 
-		Photo macro_shot = db.Photos.Create (DateTime.Now.ToUniversalTime (), "/home/ettore/Photos/macro_shot.jpg",
+		Photo macro_shot = db.Photos.Create (DateTime.Now.ToUniversalTime (), 2, "/home/ettore/Photos/macro_shot.jpg",
 						     out unused_thumbnail);
 		db.Dispose ();
 
