@@ -29,7 +29,7 @@ public enum PixbufOrientation {
 	LeftBottom = 8
 }
 
-class PixbufUtils {
+public class PixbufUtils {
 		
 	static public PixbufOrientation Rotate270 (PixbufOrientation orientation)
 	{
@@ -56,7 +56,14 @@ class PixbufUtils {
 		return orientation;
 	}
 
-	public static Pixbuf ErrorPixbuf = PixbufUtils.LoadFromAssembly ("f-spot-question-mark.png");
+	static Pixbuf error_pixbuf = null;
+	public static Pixbuf ErrorPixbuf {
+		get {
+			if (error_pixbuf == null)
+				error_pixbuf = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, "f-spot-question-mark", 256, (Gtk.IconLookupFlags)0);
+			return error_pixbuf;
+		}
+	}
 	public static Pixbuf LoadingPixbuf = PixbufUtils.LoadFromAssembly ("f-spot-loading.png");
 
 	public static int GetSize (Pixbuf pixbuf)
@@ -163,11 +170,9 @@ class PixbufUtils {
 		
 		public Pixbuf LoadFromFile (string path)
 		{
-			FileStream fs = null;
-			
 			try {
 				orientation = GetOrientation (path);
-				using (fs = File.OpenRead (path)) {
+				using (FileStream fs = File.OpenRead (path)) {
 					return Load (fs, orientation);
 				}
 			} catch (Exception) {
@@ -296,6 +301,18 @@ class PixbufUtils {
 		stream.Write (data, 0, data.Length);
 	}
 
+	static string [] NullTerminateArray (string [] options)
+	{
+		string [] terminated_options = options;
+
+		if (options != null && options [ options.Length - 1 ] != null) {
+			terminated_options = new string [options.Length + 1];
+			Array.Copy (options, terminated_options, options.Length);
+		}
+		
+		return terminated_options;
+	}
+
 	[DllImport("libgdk_pixbuf-2.0-0.dll")]
 	static extern bool gdk_pixbuf_save_to_bufferv (IntPtr raw, out IntPtr data, out IntPtr length, 
 						       string type, 
@@ -307,33 +324,14 @@ class PixbufUtils {
 		IntPtr error = IntPtr.Zero;
 		IntPtr data;
 		IntPtr length;
-		string [] terminated_options = options;
-		string [] terminated_values = values;
-
-		if (options != null && options [ options.Length - 1 ] != null) {
-			terminated_options = new string [options.Length + 1];
-			Array.Copy (options, terminated_options, options.Length);
-		}
-
-		if (values != null && values [ values.Length - 1] != null) {
-			terminated_values = new string [values.Length + 1];
-			Array.Copy (values, terminated_values, values.Length);
-		}
-
-#if false		
-		if (terminated_options != null)
-			System.Console.WriteLine ("options = {0}, values = {1}", terminated_options [0], terminated_values [0]);
-		else
-			Console.WriteLine ("no options");
-#else
 
 		bool success = gdk_pixbuf_save_to_bufferv (pixbuf.Handle, 
-							     out data, 
-							     out length, 
-							     type,
-							     terminated_options,
-							     terminated_values,
-							     out error);
+							   out data, 
+							   out length, 
+							   type,
+							   NullTerminateArray (options),
+							   NullTerminateArray (values),
+							   out error);
 		
 		if (error != IntPtr.Zero) 
 			throw new GLib.GException (error);
@@ -344,14 +342,14 @@ class PixbufUtils {
 		byte [] content = new byte [(int)length];
 		Marshal.Copy (data, content, 0, (int)length);
 
+		GLib.Marshaller.Free (data);
+
 		return content;
-#endif
 	}
 	
 	public static Pixbuf TagIconFromPixbuf (Pixbuf source)
 	{
-		// FIXME 50x50 crashes Pixdata.Serialize... what a mess.
-		int size = 52;
+		int size = (int) Tag.IconSize.Large;
 		Pixbuf tmp = null;
 		Pixbuf icon = null;
 
@@ -422,7 +420,7 @@ class PixbufUtils {
 	public static void SaveAtomic (Gdk.Pixbuf src, string filename, string type, string [] keys, string [] values)
 	{
 			string tmpname = filename + ".tmp";
-			src.Savev (tmpname, type, keys, values);
+			src.Savev (tmpname, type, NullTerminateArray (keys), NullTerminateArray (values));
 			if (rename (tmpname, filename) < 0)
 				throw new Exception ("Error renaming file");
 	}
@@ -546,26 +544,6 @@ class PixbufUtils {
 			throw new System.Exception ("Error Saving File");
 	}
 
-	[DllImport ("libgtk-win32-2.0-0.dll")]
-	extern static IntPtr gtk_icon_theme_get_default ();
-
-	[DllImport ("libgtk-win32-2.0-0.dll")]
-	extern static IntPtr gtk_icon_theme_load_icon (IntPtr theme, string name, int size, int flags, IntPtr error);
-	
-	public static Gdk.Pixbuf LoadThemeIcon (string name, int size)
-	{
-		try {
-			IntPtr native = gtk_icon_theme_load_icon (gtk_icon_theme_get_default (), name, size, 0, IntPtr.Zero);
-			if (native != IntPtr.Zero) {
-				Gdk.Pixbuf ret = (Gdk.Pixbuf) GLib.Object.GetObject(native, true);
-				return ret;
-			}
-		} catch (System.Exception e) {
-			System.Console.Write (e.ToString ());
-		}
-		return null;
-	}
-
 
 	[DllImport ("libfspot")]
 	static extern IntPtr f_pixbuf_unsharp_mask (IntPtr src, double radius, double amount, double threshold);
@@ -749,14 +727,16 @@ class PixbufUtils {
 		byte [] thumb_data = data.Data;
 		if (thumb_data.Length > 0) {
 			PixbufOrientation orientation = GetOrientation (data);
-			MemoryStream mem = new MemoryStream (thumb_data);
-			Gdk.Pixbuf thumb = new Gdk.Pixbuf (mem);
-			Gdk.Pixbuf rotated = PixbufUtils.TransformOrientation (thumb, orientation);
 			
-			if (rotated != thumb)
-				thumb.Dispose ();
-			
-			return rotated;
+			using (MemoryStream mem = new MemoryStream (thumb_data)) {
+				Gdk.Pixbuf thumb = new Gdk.Pixbuf (mem);
+
+				Gdk.Pixbuf rotated = PixbufUtils.TransformOrientation (thumb, orientation);
+				
+				if (rotated != thumb)
+					thumb.Dispose ();
+				return rotated;
+			}			
 		}
 		return null;
 	}
@@ -777,8 +757,9 @@ class PixbufUtils {
 	
 	public static PixbufOrientation GetOrientation (string path)
 	{
-		FSpot.ImageFile img = FSpot.ImageFile.Create (path);
-		return img.Orientation;
+		using (FSpot.ImageFile img = FSpot.ImageFile.Create (path)) {
+			return img.Orientation;
+		}
 	}
 
 	[DllImport("libgnomeui-2-0.dll")]
