@@ -1,3 +1,13 @@
+/*
+ * TagCommand.cs
+ *
+ * Author(s):
+ * 	Larry Ewing <lewing@novell.com>
+ * 	Stephane Delcroix <stephane@delcroix.org>
+ *
+ * This is free software. See COPYING for details
+ */
+
 using Gtk;
 using GtkSharp;
 using System;
@@ -71,15 +81,15 @@ public class TagCommands {
 
 		private void Update ()
 		{
-			if (tag_name_entry.Text == "") {
+			if (tag_name_entry.Text == String.Empty) {
 				ok_button.Sensitive = false;
-				already_in_use_label.Markup = "";
+				already_in_use_label.Markup = String.Empty;
 			} else if (TagNameExistsInCategory (tag_name_entry.Text, tag_store.RootCategory)) {
 				ok_button.Sensitive = false;
 				already_in_use_label.Markup = "<small>" + Catalog.GetString ("This name is already in use") + "</small>";
 			} else {
 				ok_button.Sensitive = true;
-				already_in_use_label.Markup = "";
+				already_in_use_label.Markup = String.Empty;
 			}
 		}
 
@@ -198,16 +208,16 @@ public class TagCommands {
 		{
 			string name = tag_name_entry.Text;
 
-			if (name == "") {
+			if (name == String.Empty) {
 				ok_button.Sensitive = false;
-				already_in_use_label.Markup = "";
+				already_in_use_label.Markup = String.Empty;
 			} else if (TagNameExistsInCategory (name, db.Tags.RootCategory)
 				   && String.Compare(name, orig_name, true) != 0) {
 				ok_button.Sensitive = false;
 				already_in_use_label.Markup = "<small>" + Catalog.GetString ("This name is already in use") + "</small>";
 			} else {
 				ok_button.Sensitive = true;
-				already_in_use_label.Markup = "";
+				already_in_use_label.Markup = String.Empty;
 				last_valid_name = tag_name_entry.Text;
 			}
 		}
@@ -296,9 +306,8 @@ public class TagCommands {
 				try {
 					t.Name = last_valid_name;
 					t.Category = categories [category_option_menu.History] as Category;
-					t.Icon = icon_image.Pixbuf;
 
-					db.Tags.Commit (t);
+					db.Tags.Commit (t, orig_name != t.Name);
 					success = true;
 				} catch (Exception ex) {
 					// FIXME error dialog.
@@ -322,7 +331,9 @@ public class TagCommands {
 		Gtk.Window parent_window;
 		FSpot.PhotoQuery query;
 		FSpot.PhotoImageView image_view;
-		IconView icon_view;
+		Gtk.IconView icon_view;
+		ListStore icon_store;
+		string icon_name = null;
 
 		[Glade.Widget] Gtk.Image preview_image;
 		[Glade.Widget] ScrolledWindow photo_scrolled_window;
@@ -330,6 +341,24 @@ public class TagCommands {
 		[Glade.Widget] Label photo_label;
 		[Glade.Widget] SpinButton photo_spin_button;
 
+		private Gdk.Pixbuf PreviewPixbuf {
+			get { return preview_image.Pixbuf; }
+			set {
+				icon_name = null;
+				preview_image.Pixbuf = value;
+			}
+
+		}
+
+		private string IconName {
+			get { return icon_name; }
+			set {
+				icon_name = value;	
+				preview_image.Pixbuf = GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, value, 48, (IconLookupFlags) 0);
+			}
+			
+		}
+
 		public FSpot.BrowsablePointer Item {
 			get {
 				return image_view.Item;
@@ -349,18 +378,18 @@ public class TagCommands {
 			Gdk.Pixbuf tmp = null;
 		       
 			image_view.GetSelection (out x, out y, out width, out height);
-			if (width > 0 && height > 0) 
-				icon_view.Selection.Clear ();
+//			if (width > 0 && height > 0) 
+//				icon_view.Selection.Clear ();
 				
 			if (image_view.Pixbuf != null) {
 				if (width > 0 && height > 0) {
 					tmp = new Gdk.Pixbuf (image_view.Pixbuf, x, y, width, height);
 					
-					preview_image.Pixbuf = PixbufUtils.TagIconFromPixbuf (tmp);
+					PreviewPixbuf = PixbufUtils.TagIconFromPixbuf (tmp);
 					
 					tmp.Dispose ();
 				} else {
-					preview_image.Pixbuf = PixbufUtils.TagIconFromPixbuf (image_view.Pixbuf);
+					PreviewPixbuf = PixbufUtils.TagIconFromPixbuf (image_view.Pixbuf);
 				}
 			}
 		}
@@ -374,37 +403,37 @@ public class TagCommands {
 			photo_spin_button.Value = item + 1;
 		}
 
-		public void HandleIconViewSelectionChanged (FSpot.IBrowsableCollection collection) 
+		public void HandleIconSelectionChanged (object o, EventArgs args)
 		{
-			// FIXME this handler seems to be called twice for each selection change
-			if (icon_view.Selection.Count > 0)
-			{
-				FSpot.IBrowsableItem item = icon_view.Selection [0];
-				string path = item.DefaultVersionUri.LocalPath;
-				try {
-					preview_image.Pixbuf = new Gdk.Pixbuf (path);
-					image_view.UnsetSelection ();
-				} catch {
-					// FIXME add a real exception handler here.
-					System.Console.WriteLine ("Unable To Load image");
-				}
-			}
+			TreeIter iter;
+			icon_store.GetIter (out iter, icon_view.SelectedItems [0]); 
+			IconName = (string) icon_store.GetValue (iter, 0);
 		}
 
+		public bool FillIconView ()
+		{
+			icon_store.Clear ();
+			string [] icon_list = FSpot.Global.IconTheme.ListIcons ("Emblems");
+			foreach (string item_name in icon_list)
+				icon_store.AppendValues (item_name, GtkUtil.TryLoadIcon (FSpot.Global.IconTheme, item_name, 32, (IconLookupFlags) 0));
+			return false;
+		}
+
+
 		public bool Execute (Tag t)
 		{
 			this.CreateDialog ("edit_icon_dialog");
 
 			this.Dialog.Title = String.Format (Catalog.GetString ("Edit Icon for Tag {0}"), t.Name);
 
-			preview_image.Pixbuf = t.Icon;
+			PreviewPixbuf = t.Icon;
 
 			query = new FSpot.PhotoQuery (db.Photos);
 			
 			if (db.Tags.Hidden != null)
-				query.Tags = new Tag [] { t, db.Tags.Hidden };
+				query.Terms = FSpot.Query.OrTerm.FromTags (new Tag []{ t, db.Tags.Hidden });
 			else 
-				query.Tags = new Tag [] { t };
+				query.Terms = new FSpot.Query.Literal (t);
 
 			image_view = new FSpot.PhotoImageView (query);
 			image_view.SelectionXyRatio = 1.0;
@@ -425,36 +454,32 @@ public class TagCommands {
 				photo_spin_button.Sensitive = false;
 				photo_spin_button.Value = 0.0;
 			}			
-			
-			
-			// FIXME this path choosing method is completely wrong/broken/evil it needs to be
-			// based on real data but I want to get this release out.
-			string theme_dir = System.IO.Path.Combine (FSpot.Defines.GNOME_ICON_THEME_PREFIX,
-								   "share/icons/gnome/48x48/emblems");
-			if (System.IO.Directory.Exists (theme_dir))
-				icon_view = new IconView (new FSpot.DirectoryCollection (theme_dir));
-			else if (System.IO.Directory.Exists ("/opt/gnome/share/icons/gnome/48x48/emblems"))
-				icon_view = new IconView (new FSpot.DirectoryCollection ("/opt/gnome/share/icons/gnome/48x48/emblems"));
-			else if (System.IO.Directory.Exists ("/usr/share/icons/gnome/48x48/emblems"))
-				icon_view = new IconView (new FSpot.DirectoryCollection ("/usr/share/icons/gnome/48x48/emblems"));
-			else // This will just load an empty collection if the directory doesn't exist.
-				icon_view = new IconView (new FSpot.DirectoryCollection ("/usr/local/share/icons/gnome/48x48/emblems"));
+
+			icon_store = new ListStore (typeof (string), typeof (Gdk.Pixbuf));
+
+			icon_view = new Gtk.IconView (icon_store); 
+			icon_view.PixbufColumn = 1;
+			icon_view.SelectionMode = SelectionMode.Single;
+			icon_view.SelectionChanged += HandleIconSelectionChanged;
 
 			icon_scrolled_window.Add (icon_view);
-			icon_view.ThumbnailWidth = 32;
-			icon_view.DisplayTags = false;
-			icon_view.DisplayDates = false;
-			icon_view.Selection.Changed += HandleIconViewSelectionChanged;
+
 			icon_view.Show();
 
 			image_view.Show ();
 
+			FSpot.Delay fill_delay = new FSpot.Delay (FillIconView);
+			fill_delay.Start ();
+
 			ResponseType response = (ResponseType) this.Dialog.Run ();
 			bool success = false;
 
 			if (response == ResponseType.Ok) {
 				try {
-					t.Icon = preview_image.Pixbuf;
+					if (IconName != null)
+						t.ThemeIconName = IconName;
+					else
+						t.Icon = PreviewPixbuf;
 					//db.Tags.Commit (t);
 					success = true;
 				} catch (Exception ex) {
